/*************************** Check and Radio buttons * */
popover.combo list { min-width: 200px; }

window.csd.unified:not(.solid-csd), window.csd.unified:not(.solid-csd) headerbar { border-radius: 0; }

.windowhandle, .windowhandle * { -GtkWidget-window-dragging: true; }
