/*************************** Check and Radio buttons * */
row label.subtitle { font-size: smaller; opacity: 0.55; text-shadow: none; }

row > box.header { margin-left: 12px; margin-right: 12px; min-height: 50px; }

row > box.header > box.title { margin-top: 8px; margin-bottom: 8px; }

row.expander { background-color: transparent; }

row.expander list.nested > row { background-color: alpha(#f6f5f4, 0.5); border-color: alpha(#cdc7c2, 0.7); border-style: solid; border-width: 1px 0px 0px 0px; }

row.expander image.expander-row-arrow { transition: 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94); }

row.expander:checked image.expander-row-arrow { -gtk-icon-transform: rotate(0turn); }

row.expander:not(:checked) image.expander-row-arrow { opacity: 0.55; text-shadow: none; }

row.expander:not(:checked) image.expander-row-arrow:dir(ltr) { -gtk-icon-transform: rotate(-0.25turn); }

row.expander:not(:checked) image.expander-row-arrow:dir(rtl) { -gtk-icon-transform: rotate(0.25turn); }

row.expander:checked image.expander-row-arrow:not(:disabled) { color: #3584e4; }

row.expander image.expander-row-arrow:disabled { color: #929595; }

deck > dimming, leaflet > dimming { background: rgba(0, 0, 0, 0.12); }

deck > border, leaflet > border { min-width: 1px; min-height: 1px; background: rgba(0, 0, 0, 0.05); }

deck > shadow, leaflet > shadow { min-width: 56px; min-height: 56px; }

deck > shadow.left, leaflet > shadow.left { background-image: linear-gradient(to right, rgba(0, 0, 0, 0.05), rgba(0, 0, 0, 0.01) 40px, rgba(0, 0, 0, 0) 56px), linear-gradient(to right, rgba(0, 0, 0, 0.03), rgba(0, 0, 0, 0.01) 7px, rgba(0, 0, 0, 0) 24px); }

deck > shadow.right, leaflet > shadow.right { background-image: linear-gradient(to left, rgba(0, 0, 0, 0.05), rgba(0, 0, 0, 0.01) 40px, rgba(0, 0, 0, 0) 56px), linear-gradient(to left, rgba(0, 0, 0, 0.03), rgba(0, 0, 0, 0.01) 7px, rgba(0, 0, 0, 0) 24px); }

deck > shadow.up, leaflet > shadow.up { background-image: linear-gradient(to bottom, rgba(0, 0, 0, 0.05), rgba(0, 0, 0, 0.01) 40px, rgba(0, 0, 0, 0) 56px), linear-gradient(to bottom, rgba(0, 0, 0, 0.03), rgba(0, 0, 0, 0.01) 7px, rgba(0, 0, 0, 0) 24px); }

deck > shadow.down, leaflet > shadow.down { background-image: linear-gradient(to top, rgba(0, 0, 0, 0.05), rgba(0, 0, 0, 0.01) 40px, rgba(0, 0, 0, 0) 56px), linear-gradient(to top, rgba(0, 0, 0, 0.03), rgba(0, 0, 0, 0.01) 7px, rgba(0, 0, 0, 0) 24px); }

avatar { border-radius: 9999px; -gtk-outline-radius: 9999px; font-weight: bold; }

avatar.color1 { background-image: linear-gradient(#ffbe6f, #ed6f00); color: #ffe5c5; }

avatar.color2 { background-image: linear-gradient(#f8e45c, #e5a50a); color: #f9f4e1; }

avatar.color3 { background-image: linear-gradient(#dc8add, #8a3ea3); color: #e7c2e8; }

avatar.color4 { background-image: linear-gradient(#99c1f1, #337fdc); color: #cfe1f5; }

avatar.color5 { background-image: linear-gradient(#c0bfbc, #6e6d71); color: #d8d7d3; }

avatar.color6 { background-image: linear-gradient(#8de6ae, #29ae71); color: #cbf7d5; }

avatar.color7 { background-image: linear-gradient(#f67365, #d91a23); color: #f6c8c4; }

avatar.color8 { background-image: linear-gradient(#cdab8f, #865d3c); color: #e5d6ca; }

avatar.contrasted { color: #fff; }
